#!/bin/sh

#################
# SHELL OPTIONS #
#################

# make wildcard globbing with no match return null instead of literal
shopt -s nullglob
# make wildcard globbing case-insensitive (such as WAV vs. wav)
shopt -s nocaseglob

########################
# FUNCTION DEFINITIONS #
########################

# function to wait for user interaction
pause()
{
	echo "Press any key to continue . . ."
	while [ true ] ; do
		# poll keyboard every second to read one character
		# suppresses echo of character to console
		read -s -t 1 -n 1
		if [ $? = 0 ] ; then
			# quit loop when user actually makes an entry
			break;
		fi
	done
}

# function to run any time script needs to abort
enditall()
{
	pause
	# close terminal without user intervention
	osascript -e 'tell application "Terminal" to close (every window whose name contains "MultiEFE")' & exit 1
}

###############
# MAIN SCRIPT #
###############

# change terminal window name
echo -n -e "\033]0;MultiEFE\007"

# change terminal window dimensions
echo -n -e "\033[8;40;80t"

# get directory that launched script and use it as current directory
DIR=$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )
cd "${DIR}"

# create the [EFE_IN] directory (with error suppression if already exists)
mkdir ./\[EFE_IN\] 2>/dev/null

clear
echo "##################################################"
echo "# Ensoniq MultiEFE to IMG and HFE Converter v0.7 # (EPS/EPS16 800K)"
echo "##################################################"
echo ""
echo "Input files must be Ensoniq instruments (EFE) with either *.efe or *.ins file "
echo "extensions, as generated by various Ensoniq utilities such as EPSDisk, Awave, "
echo "EpsLin, EnsoniqFS, and Translator."
echo ""
echo "EFEs must be placed in the [EFE_IN] folder. Multiple disks can be created at "
echo "once by using subfolders within the root of the [EFE_IN] folder."
echo ""
echo "In order to maximize storage, there will be no operating system, so the "
echo "resulting disk images will not be bootable."
echo ""
echo "Additionally, any created IMGs will automatically be converted into EPS/EPS16 "
echo "compatible HFEs which can be used with floppy emulators."
echo ""
pause
echo ""

# check for required tools and metadata
if [ ! -f "./tools/epslin" ]; then
	echo "Critical Error: epslin cannot be found. Aborting! \n"
	enditall
fi

if [ ! -f "./tools/App/hxcfe" ]; then
	echo "Critical Error: hxcfe cannot be found. Aborting! \n"
	enditall
fi

if [ ! -f "./tools/Frameworks/libhxcfe.dylib" ]; then
	echo "Critical Error: libhxcfe cannot be found. Aborting! \n"
	enditall
fi

if [ ! -f "./tools/Frameworks/libusbhxcfe.dylib" ]; then
	echo "Critical Error: libusbhxcfe cannot be found. Aborting! \n"
	enditall
fi

# check for number of EFE/efe/INS/ins files in the [EFE_IN] folder
# note: done recursively so sub-directories are supported
foundEFEs=$(find ./\[EFE_IN\] -type f \( -iname '*.efe' -o -iname '*.ins' \) | wc -l)
if [ $foundEFEs -eq 0 ]; then
	echo "Critical Error: No EFE or INS files found to convert. Aborting! \n"
	enditall
fi

# process root directory and any sub-folders one level deep
find ./\[EFE_IN\] -maxdepth 1 -type d -print0 |
while IFS= read -r -d '' ensfolder; do
	OLDIFS=$IFS
	IFS=$'\n'
	# get subfolder name only without rest of the path
	ensfolder=$(basename -- "$ensfolder")

	# check if root folder is being processed or not
	if [ "$ensfolder" = "[EFE_IN]" ]; then
		# processing root folder
		ensfolder=""
		targetName="ROOT"
	else
		# processing a sub-folder
		targetName="$ensfolder"
	fi # end of root detection

	imgsize=$(find ./\[EFE_IN\]/"$ensfolder" -maxdepth 1 -type f \( -iname '*.efe' -o -iname '*.ins' \) -exec ls -l {} \; | awk '{insize += $5} END{print insize}')

	# empty folders should skip EFE storage and HFE/IMG creation entirely
	doProcess=1
	# when imgsize is undefined then the folder is empty, so do not process it
	if [ -z ${imgsize} ]; then
		echo "Empty folder found. Skipping... \n"
		# flag current folder for a skip
		doProcess=0
	fi

	# skip control
	if [ $doProcess -eq 1 ]; then
		# check that EFE files do not exceed EPS blank capacity
		if [ $imgsize -gt 811520 ]; then
			oversized=$(($imgsize-811520))
			echo "Warning: EFE files are too large to fit on a single EPS DD disk. "
			echo "Only 811,520 bytes allowed: $oversized bytes too many. \n"
			echo "Process will continue, but at least one EFE will be skipped. "
			echo ""
		fi
		IFS=$OLDIFS

		echo "Storing EFEs to IMG... \n"

		# create IMG output folder, if needed
		if [ ! -d "./[IMG_EPS_OUT]" ]; then
			mkdir ./\[IMG_EPS_OUT\]
		fi

		# create new blank disk image in EPS 800K format
		./tools/epslin -q -l DISK000 -fi "./[IMG_EPS_OUT]/${targetName}.IMG" eps > /dev/null

		# store all found EFE/efe/INS/ins files in selected folder to new disk image
		find ./\[EFE_IN\]/"$ensfolder" -maxdepth 1 -type f \( -iname '*.efe' -o -iname '*.ins' \) -print0 |
		while IFS= read -r -d '' ensfile; do
				# strip path to derive filename only (without path or extension)
				ensname=$(basename -- "$ensfile")
				ensname="${ensname%.*}"
				./tools/epslin -p1 "./[IMG_EPS_OUT]/${targetName}.IMG" "${ensfile}" > /dev/null 2>&1
				# check errorlevel and report success or failure
				if [ "$?" = 0 ]; then
					# success
					echo "$ensname was added to the disk image. "
				else
					# failure
					echo "Warning: $ensname could not be written to the disk image. "
				fi # end of errorlevel routine
			done # end of input file processing

			# generate report of final IMG
			./tools/epslin -g0 "./[IMG_EPS_OUT]/${targetName}.IMG"

			# create HFE output folder, if needed
			if [ ! -d "./[HFE_EPS_OUT]" ]; then
				mkdir ./\[HFE_EPS_OUT\]
			fi

			echo "Generating EPS-type HFE... "
			./tools/App/hxcfe -finput:"./[IMG_EPS_OUT]/${targetName}.IMG" -conv -uselayout:ENSONIQ_DD_800KB -ifmode:GENERIC_SHUGART_DD_FLOPPYMODE -foutput:"./[HFE_EPS_OUT]/${targetName}.hfe" > /dev/null
			echo ""
		fi # end of skip control

done # end of directory processing

echo "Process complete. \n"
echo "EPS compatible IMG and HFE files should now be present within the "
echo "[IMG_EPS_OUT] and [HFE_EPS_OUT] folders. \n"
pause

# close terminal without user intervention
osascript -e 'tell application "Terminal" to close (every window whose name contains "MultiEFE")' & exit 0
